
import requests
import json
import csv
import time
import pandas as pd
from shapely.geometry import shape
from shapely import wkt

names = []
url = "https://nominatim.openstreetmap.org/search"
extra_cols = {"place_id": [], 
              "lat": [], 
              "lon": [], 
              "shelter_bbox": [], 
              "6_min_isochrone": [], 
              "10_min_isochrone": []
              }

with open("clim_shelter.csv", newline="", encoding="utf-8") as f:
    reader = csv.DictReader(f)
    i = 1
    for row in reader:
        valhalla_url = "http://localhost:8002/isochrone"
        place_name = row["name_es"]
        query = f"{place_name}, Bilbao, Spain"
        params = {"q": query, "format": "json", "limit": 1}
        response = requests.get(url, params=params, headers = {"User-Agent": "streamlit-region-app"})
        print (response.json())
        lat = response.json()[0]["lat"] if response.json() else None
        lon = response.json()[0]["lon"] if response.json() else None   
        shelter_bbox = response.json()[0]["boundingbox"] if response.json() else None
        place_id = response.json()[0]["place_id"] if response.json() else None
        print (lat, lon, shelter_bbox, place_id)

        # JSON request for Zubizuri Bridge, Bilbao
        payload = {
            "locations": [
                {
                    "lat": lat,
                    "lon": lon,
                    "name": place_name
                }
            ],
            "costing": "pedestrian",
            "contours": [
                {"time": 6},
                {"time": 10}
            ],
            "polygons": True,
            "denoise": 1, ## We don't want very fine geometry, since we would be keeping this in a STAC
            "generalize": 50 ## We don't want very fine geometry, since we would be keeping this in a STAC
        }

        if lat is not None and lon is not None:
            print (f"Requesting Isochrone for {place_name} at Lat: {lat}, Lon: {lon}")
        else:
            extra_cols["place_id"].append(None)
            extra_cols["lat"].append(None)
            extra_cols["lon"].append(None)
            extra_cols["shelter_bbox"].append(None)
            extra_cols["6_min_isochrone"].append(None)
            extra_cols["10_min_isochrone"].append(None)
            continue

        six_min_isochrone, ten_min_isochrone = None, None
        response = requests.post(valhalla_url, json=payload)
        geojson_data = response.json()

        if response.status_code != 200:
            print (f"Error in Valhalla API Request for {place_name}, Request failed with Status Code: {response.status_code}")
        else: 
            six_min_isochrone, ten_min_isochrone = shape(geojson_data["features"][0]["geometry"]).wkt,  shape(geojson_data["features"][1]["geometry"]).wkt 


        if lat and lon and shelter_bbox:
            extra_cols["place_id"].append(place_id)
            extra_cols["lat"].append(lat)
            extra_cols["lon"].append(lon)
            extra_cols["shelter_bbox"].append(shelter_bbox)
            extra_cols["6_min_isochrone"].append(six_min_isochrone)
            extra_cols["10_min_isochrone"].append(ten_min_isochrone)

        print (f"Processed {i} records")
        print ("-----------------------------")
        i += 1
        time.sleep(2) ## To avoid hitting the rate limit of Nominatim API

df = pd.read_csv("clim_shelter.csv")

# Add new columns
for col_name, values in extra_cols.items():
    df[col_name] = values

# Save
df.to_csv("clim_shelter_enriched.csv", index=False)


from klab_nifi_py import * 


space = Space(
    shape= "POLYGON(( 88.2660 22.4492, 88.4611 22.4492, 88.4611 22.6823, 88.2660 22.6823, 88.2660 22.4492))"
)

dt_2020 = datetime(2020, 1, 1, 0, 0, 0)
dt_2021 = datetime(2021, 12, 31, 23, 59, 59)


time = Time(
    tstart=dt_2020,
    tend = dt_2021
    )


persistantResourceConfig = contextualizer.PersistentResource(
    mode = contextualizer.PersistentResource.ResourceUpdateMode.UPDATE,
    namespace="klab.nifi.internal.tests.resources",
    service="im.resources-main",
    catalog ="staging",
    id="slope",
)



ctx = contextualizer.WCS(
    #wcsIdentifier="im-data-global-geography__elevation-global-90m",
    wcsIdentifier="im-data-global-geography__slope-global-90m",
    resource=persistantResourceConfig
)

stacCtx = contextualizer.STAC(
    collection="https://planetarycomputer.microsoft.com/api/stac/v1/collections/landsat-c2-l2",
    asset="red",

    
)


#im.resources-main:staging:klab.nifi.internal.tests.resources:slope


klabNifiObs = KlabObservationNifiRequest(
    space = space,
    time  = time,
    ##resetContext=True,
    ##observationSemantics= "geography:Slope",
    observationSemantics= "earth:Terrestrial earth:Region",
    observationNamespace="nifi.internal.tests",
    asContext=True,
    observationName="WB",
    dtURL="https://services.integratedmodelling.org/runtime/main/api/v1/dt/ESA_INSTITUTIONAL.rvr3s2juw0",
    #contextualizer=ctx
)

print (klabNifiObs.to_dict())

print (klabNifiObs.to_json())

nifiklabClient = KlabNifiListenHTTPClient(port="3306", healthport="3307")
nifiklabClient.submitObservation(klabNifiObs)




from pydantic.dataclasses import dataclass
from dataclasses import field
from .commons import BaseModel, KLAB_SERVICETYPE_KEY, KLAB_DEFAULT_WCS_URL
from .logging import logger


@dataclass
class PersistentResource(BaseModel):
    '''
    Persistent Resource for Contextualization

    :param service: [Required] The Service which has the Resource first
    :param catalog: [Required] The Catalog where the resource is to be located
    :param id: [Required] Resource ID
    :param namespace: [Optional, Default: "im:nifi"] Namespace for the Persistent Resource
    :param mode: [Optional, Default: UPDATE] Persistent Resource Update Mode
    '''

    class ResourceUpdateMode:
        '''
        Persistent Resource Update Modes
        Modes:
            UPDATE: Update the existing Persistent Resource with new Contextualization
            REPLACE: Replace the existing Persistent Resource with new Contextualization
            MERGE: Merge the new Contextualization with existing Persistent Resource
            ADD: Add new Contextualization as a new Persistent Resource
        '''
        UPDATE = "update"
        REPLACE = "replace"
        MERGE = "merge"
        ADD = "add"

    service: str ## The Resource Service that has the resource
    catalog: str 
    id: str
    namespace: str
    mode: str = ResourceUpdateMode.UPDATE #Default: Update


@dataclass(kw_only=True)
class Contextualizer(BaseModel):
    '''
    Base Contextualizer Class to be inherited all the Contextualizers
    :param serviceType: [Internal] Service Type of the Contextualizer, internally set, not to be set by user
    '''
    serviceType: str = field(init=False, default=None)
    persistent: bool = field(default=False, kw_only=True)
    resource: PersistentResource = field(default=False, kw_only=True) ## Only if persistent is True

    def __setattr__(self, name, value):
        if name == KLAB_SERVICETYPE_KEY and hasattr(self, KLAB_SERVICETYPE_KEY):
            raise AttributeError("serviceType is read-only and set internally")
        super().__setattr__(name, value)


    def persistantConfigCheck(self):
        '''
        If the Resource Configuration is provided, set the Persistent Flag to True
        even if the User has not set the Persistent Flag to True or has set it to False
        This is to ensure that the Contextualization is always Persistent if the Resource
        '''

        if self.resource is not None:
            logger.info("Setting Persistance basis the Resource Configuration Provided")
            self.persistent = True
        else:
            logger.info("No Resource Configuration Provided, Setting Persistance to False")
            self.persistent = False

@dataclass(kw_only=True)
class WCS(Contextualizer):

    '''
    WCS (Web Coverage Service) contextualization

    :param wcsIdentifier: [Required] WCS Coverage Identifier
    :param band: [Optional, Default: 0] Band Number (For Multi Band Coverages. Handling Single Band by Default)
    :param wcsVersion: [Optional, Default: "2.0.1"] WCS Version
    :param serviceUrl: [Optional, Default: "https://integratedmodelling.org/geoserver/ows"] WCS Service URL
    :param persistent: [Optional, Default: False] Whether the Contextualization is to be Persistent Resource
    '''
    
    wcsIdentifier: str
    band: int = 0 ## Handling Single Band by Default
    wcsVersion: str = "2.0.1" ## Default WCS Version
    serviceUrl: str = KLAB_DEFAULT_WCS_URL ## Referring to the IM GeoServer by default
    resource: PersistentResource = None ## Only if persistent is True

    def __post_init__(self):
        object.__setattr__(self, KLAB_SERVICETYPE_KEY, "wcs")
        self.persistantConfigCheck()


@dataclass(kw_only=True)
class STAC(Contextualizer):
    '''
    STAC (SpatioTemporal Asset Catalog) contextualization

    :param collection: [Required] STAC Collection URL
    :param asset: [Required] STAC Asset Name
    :param band: [Optional, Default 0] Band Number (For Multi Band CoGs. This is usually unusual)
    :param persistent: [Optional, Default: False] Whether the Contextualization is to be Persistent Resource
    '''

    collection: str
    asset: str
    band: int = 0
    resource: PersistentResource = None

    def __post_init__(self):
        object.__setattr__(self, KLAB_SERVICETYPE_KEY, "stac")
        self.persistantConfigCheck()




